/*
 * File         : BV_Tango_Control.C
 * Author       : Tomasz Stachowiak
 * Created      : Thu Oct 22 14:28:10 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include "BVN/SprocThread.h"
#include "BV_Session_Control.h"
#include "tango.h"
#include "BV_Tango_Control.h"

//#define _DEBUG

BV_Tango_Control::BV_Tango_Control(int argc, char **argv)
{
  session_ = BV_Session_Control::instance();

  tango_sock_ = registerApp(argc, argv);
}

BV_Tango_Control::~BV_Tango_Control()
{
  close(tango_sock_);
  session_->destroy();
}

int BV_Tango_Control::handle_events()
{
  char *msg;
  struct Event *event;
  int no;
  Participant part;
  BV_Member *mbr;
  fd_set set;
  struct timeval timeout;
  static int i = 0;

  if(tango_sock_ == NULL)
    return -1;

  timeout.tv_sec = 0;
  timeout.tv_usec = 100000;

  FD_ZERO(&set);
  FD_SET(tango_sock_, &set);

  if(select(FD_SETSIZE, &set, 0, 0, &timeout) > 0)
    {
      //printf("select ok\n");
      //fflush(stdout);
      if(FD_ISSET(tango_sock_, &set))
	 {
	   //printf("fd set\n");
	   //fflush(stdout);
	   event = receiveEvent(tango_sock_);
	   if(event == NULL)
	     {
#ifdef _DEBUG
	       printf("BV_Tango_Control::handle_events: read TANGO msg error!\n");
	       fflush(stdout);
#endif
	       return -1;
	     }
	   
	   msg = (char *)getData(event);
	   strcpy(part.user, "user");
	   switch(msg[0])
	     {
	     case JOIN:
	       no = msg[1];
	       part.host = *(int *)&msg[2];
	       part.handle = ntohl(*(int *)&msg[6]);
	       memcpy(part.user, &msg[11], msg[10]);
	       part.user[msg[10]] = 0;
#ifdef _DEBUG
	       printf("BV_Tango_Control::handle_events: JOIN handle %d, name %s\n", 
		      part.handle, part.user);
	       fflush(stdout);
#endif
	       mbr = new BV_Member(NULL, part.handle, part.user, part.host);
	       session_->new_member(mbr);
	       break;
	       
	     case NEW:
	       part.host = *(int *)&msg[1];
	       part.handle = ntohl(*(int *)&msg[5]);
	       memcpy(part.user, &msg[10], msg[9]);
	       part.user[msg[9]] = 0;
#ifdef _DEBUG
	       printf("BV_Tango_Control::handle_events: NEW handle %d, name %s\n", 
		      part.handle, part.user);
	       fflush(stdout);
#endif
	       mbr = new BV_Member(NULL, part.handle, part.user, part.host);
	       session_->new_member(mbr);
	       break;
	       
	     case LEAVE:
#ifdef _DEBUG
	       printf("BV_Tango_Control::handle_events: LEAVE\n");
	       fflush(stdout);
#endif
	       return -1;
	       break;
	       
	     case DELETE:
	       part.host = *(int *)&msg[1];
	       part.handle = ntohl(*(int *)&msg[5]);
	       memcpy(part.user, &msg[10], msg[9]);
	       part.user[msg[9]] = 0;
#ifdef _DEBUG
	       printf("BV_Tango_Control::handle_events: DELETE handle %d, name %s\n", part.handle, part.user);
#endif
	       session_->delete_member(part.handle);
	       break;
	       
	     case 122:  
	       /* bring to front */
#ifdef _DEBUG
	       printf("BV_Tango_Control::handle_events: BRING_TO_FRONT\n");
#endif
	       break;
	       
	     default:
	       break;
	     }
	   
	   fflush(stdout);
	   
	   destroyEvent(event);
	 }
    }
	   
  //if(i == 0)
  session_->handle_member_events();
  i = (i + 1) % 50;

  return 0;
}

int BV_Tango_Control::send_msg(char *msg, int len)
{
  struct Event *event;

  event = createEvent(CONTROL, len, msg);
  sendEvent(tango_sock_, event);
  destroyEvent(event);
  
  return 0;
}

